<?php
/**
 * Created by PhpStorm.
 * User: rafaelporto
 * Date: 04/04/2016
 * Time: 11:02
 */

namespace Service;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Teknisa\Entities;
use Util\Utilities;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class CallLabels {

    /** @var \Doctrine\ORM\EntityManager $entityManager */
    protected $entityManager;
    /** @var \Util\Utilities $environment */
    protected $environment;
    /** @var \Factory\CallLabel $callLabelFactory */
    protected $callLabelFactory;
    /** @var \Factory\CallLabelRelation $callLabelRelationFactory */
    protected $callLabelRelationFactory;

    /** @var \Teknisa\Model\Teknisa\Repositories\CpCallLabel $callLabelRepository */
    protected $callLabelRepository;
    /** @var \Teknisa\Model\Teknisa\Repositories\CpCallLabelRelation $callLabelRelationRepository */
    protected $callLabelRelationRepository;

    public function __construct() {
        $this->entityManager  = InstanceProvider::getEntityManager();
        $this->environment    = InstanceProvider::getEnvironment();
        $this->callLabelFactory = InstanceProvider::getFactory()->getCallLabel();
        $this->callLabelRelationFactory = InstanceProvider::getFactory()->getCallLabelRelation();

        $this->callLabelRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpCallLabel');
        $this->callLabelRelationRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpCallLabelRelation');
    }

    public function saveNewCallLabel($description, $cdCliente) {
        $connection = $this->entityManager->getConnection();

        try {
            $this->entityManager->beginTransaction();

            $callLabel = $this->callLabelFactory->createCallLabel($cdCliente,$description);

            $this->entityManager->persist($callLabel);
            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function updateCallLabel($id, $description, $updatedBy) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();

            /** @var \Teknisa\Model\Teknisa\Entities\CpCallLabel $callLabelEntity */
            $callLabelEntity = $this->callLabelRepository->find($id);

            $callLabel = $this->callLabelFactory->updateCallLabel($callLabelEntity, $updatedBy,$description);

            $this->entityManager->persist($callLabel);
            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function deleteCallLabelRelationBy($parameters) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $callLabelRelations = $this->callLabelRelationRepository->findBy($parameters);
            foreach($callLabelRelations as $callLabelRelation)
                $this->entityManager->remove($callLabelRelation);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function deleteCallLabel($callLabelID) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $this->deleteCallLabelRelationBy(array('label' => $callLabelID));
            $callLabel = $this->callLabelRepository->find($callLabelID);
            $this->entityManager->remove($callLabel);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function findAllCallLabels($conditions,$page,$pageSize) {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $filter = Utilities::createQueryFiltered($conditions);
        if ($this->environment->currentUserIsTeknisa()) {
            $clientes = "1 = 1";
        } else {
            $operadorRepository   = InstanceProvider::getRepository()->getOperador();
            $clientes = $operadorRepository->findAvailableCliente($this->environment->getCurrentUserId(), 0);
            $clientes = DevOpsUtilities::getArrayFromArrayAssociative($clientes, "CDCLIENTE");
        }
        $clienteFilter = $clientes == "1 = 1" ? $clientes : DevOpsUtilities::fixBugWhereINQuery($clientes, "C.CDCLIENTE", false, true);
        return $this->callLabelRepository->getAllCallLabels($filter, $limit, $offset, $clienteFilter);
    }

    public function editCallRelationWithLabels($callId, $callLabelIds){
        foreach ($callLabelIds as $callLabelId) {

            if(!empty($callLabelId)){
                $callLabelRelation = $this->callLabelRelationRepository->findBy(array("label" => $callLabelId, "call" => $callId));

                if (empty($callLabelRelation)) {
                    $callLabelRelation = $this->callLabelRelationFactory->createCallLabelRelation($callId, $callLabelId);
                    $this->entityManager->persist($callLabelRelation);
                }
            }

        }
        $this->entityManager->flush();

        $currentUser = $this->environment->getCurrentUserId();
        /** @var Entities\CpCallLabelRelation[] $callLabelRelations */
        $callLabelRelations = $this->callLabelRelationRepository->findBy(array("call" => $callId, "cdoperinclusao" => $currentUser));

        foreach ($callLabelRelations as $callLabelRelation) {
            if(!in_array($callLabelRelation->getLabel(), $callLabelIds)){
                $this->entityManager->remove($callLabelRelation);
            }
        }
    }


    public function createCallRelationWithLabels($callId, array $labelIds) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            foreach($labelIds as $labelId) {
                $callLabelRelation = $this->callLabelRelationFactory->createCallLabelRelation($callId, $labelId);
                $this->entityManager->persist($callLabelRelation);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
}
