<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;
use Util\Utilities;
use Util\Messages;
use Repository\WorkItem as WorkItemRepository;

class WorkItem
{
    /**
     * @var EntityManager
     */
    private $entityManager;
    /**
     * @var WorkItemRepository
     */
    private $workItemRepository;
    /**
     * @var Utilities
     */
    private $environment;

    /**
     * WorkItem constructor.
     * @param EntityManager $entityManager
     * @param WorkItemRepository $workItemRepository
     */
    public function __construct(EntityManager $entityManager, WorkItemRepository $workItemRepository)
    {
        $this->environment = InstanceProvider::getEnvironment();
        $this->entityManager = $entityManager;
        $this->workItemRepository = $workItemRepository;
    }

    public function findWorkItem($params)
    {
        return $this->workItemRepository->find($params);
    }

    public function findLevelPattern()
    {
        return $this->workItemRepository->findLevelPattern();
    }

    public function findItemtrab()
    {
        return $this->workItemRepository->findItemtrab();
    }

    public function findMasterItemSelectItems()
    {
        return $this->workItemRepository->findMasterItemSelectItems();
    }

    public function findWorkItemType()
    {
        return $this->workItemRepository->findWorkItemType();
    }

    public function insert($params) {
        $cdOperador   = $this->environment->getCurrentCdoperador();
        $nrOrg        = $this->environment->getCurrentOrganizationId();
        $cdItemTrab   = InstanceProvider::getNewCode()->getCodeByTableName('ITEMTRAB', $nrOrg);
        $cdArvItTrab  = !empty($params['CDPARENT_ITTRAB']) ? $params['CDPARENT_ITTRAB']['CDARVITTRAB'] . $params['CDARVITTRAB'] : $params['CDARVITTRAB'];
        $cdNvItemTrab = (!empty($params['CDPARENT_ITTRAB']) ? $params['CDPARENT_ITTRAB']['CDNVITEMTRAB'] : 0) + 1;

        $itemTrab = $this->workItemRepository->findWorkItemByCodeAndLevel($cdArvItTrab, $cdNvItemTrab);
        if(!empty($itemTrab['data'])) {
            throw new \Exception(Messages::ALE_ITEM_TRAB_ALREADY_EXISTS);
        }

        $this->workItemRepository->insert($cdItemTrab, $params['NMITEMTRAB'], $cdArvItTrab, $cdNvItemTrab,
            $params['NMITTRABESPA'], $params['NMITTRABINGL'], $params['CDREDUITTRAB'], $params['CDTIPOITTRAB'], $nrOrg, $cdOperador);
    }

    public function update($params) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $cdOperador = $this->environment->getCurrentCdoperador();
            $nrOrg = $this->environment->getCurrentOrganizationId();

            foreach($params as $row) {
                $this->workItemRepository->update($row['CDITEMTRAB'], $row['NMITEMTRAB'], $row['NMITTRABESPA'], $row['NMITTRABINGL'],
                    $row['CDREDUITTRAB'], $row['CDTIPOITTRAB'], $row['IDATIVO'], $nrOrg, $cdOperador);
            }

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
}
