import { SocketClient, SocketFactory } from '@zeedhi/core';
import { InnerSocket } from '@zeedhi/core/types/sockets/inner-socket';

export class WebSocketClient extends SocketClient {
	private socketClient?: InnerSocket;

	private currentRoom?: string;

	private currentResolve: any;

	private currentReject: any;

	private currentEvent: any;

	public connectClient(url: string) {
		const promise = new Promise((resolve) => {
			if (!this.socketClient?.isOpen()) {
				this.socketClient = SocketFactory.create(url);
				this.socketClient.addListener({
					onOpen: () => {
						resolve('connected');
					},
					onMessage: (message: any) => {
						if (message.data instanceof Array || typeof message.data === 'string') {
							const errorSuccessRule = message.isError ? 'color: red;' : 'color: green;';
							console.log(this.getLogMessage(message), 'color: blue;', 'color: black;', 'color: blue;', 'color: black;', errorSuccessRule);
						}
						if (message.event === 'noOtherClientsConnected') {
							this.currentReject('clientDisconnected');
						} if (message.action === this.currentEvent) {
							if (message.isError) {
								this.currentReject(message.data);
							} else {
								this.currentResolve(message.data);
							}
						}
					},
				});
			} else {
				resolve('connected');
			}
		});

		return promise;
	}

	private getLogMessage(data: any) {
		let message = `'%cAction:%c ${data.action} | %cResponse:%c `;

		if (typeof data.data === 'object') {
			message += `["' ${data.data.join('", "')} '"] | `;
		} else {
			let value = data.data.replace(/\n/g, ' ');
			if (value !== '') {
				value = value.trim();
			}
			message += `"' ${value} '" | `;
		}

		message += data.isError ? '%cError' : '%cSuccess';

		return message;
	}

	public async loginBrowser(email: string) {
		this.joinRoom('pm_do', email);
		const privateRoom = await this.send({
			event: 'getPrivateRoom',
			data: {
				email,
			},
		});
		this.joinRoom('pm_do', privateRoom);
	}

	public joinRoom(namespace: string, email: any) {
		const eventsToStart = ['createNamespace', 'createRoom', 'joinRoom'];
		const socket = this.socketClient;
		const payload = {
			event: 'loginBrowser',
			namespace,
			room: email,
			data: {
				email,
				key: 'you can create!',
			},
		};

		eventsToStart.forEach((event: any) => {
			payload.event = event;
			socket?.send(JSON.stringify(payload));
		});
		this.currentRoom = email;
	}

	private async send(payload: any) {
		const promise = new Promise((resolve, reject) => {
			this.currentResolve = resolve;
			this.currentReject = reject;
		});
		this.currentEvent = payload.event;

		payload.room = payload.room || this.currentRoom;
		payload.namespace = payload.namespace || 'pm_do';
		this.socketClient?.send(JSON.stringify(payload));

		return promise;
	}

	public async getDirectoryFiles(path: string, filter: any) {
		const result = await this.send({
			event: 'getDirectoryFiles',
			data: {
				path,
				filter,
			},
		});
		return result;
	}
}
